from wsgiref.simple_server import WSGIServer, WSGIRequestHandler
from rivr.wsgi import WSGIHandler

def serve(handler, host='localhost', port=8080):
    httpd = WSGIServer((host, port), WSGIRequestHandler)
    httpd.set_app(WSGIHandler(handler))
    print("Server is running at http://%s:%s/" % (host, port))
    httpd.serve_forever()



from rivr.http import Http404

class Array(object):
    def __init__(self, *views):
        self.views = views
    
    def __call__(self, request):
        for view in self.views:
            try:
                response = view(request)
                if response:
                    return response
            except Http404:
                continue
        
        raise Http404



class Context(object):
    def __init__(self, kwargs={}):
        self.dicts = [kwargs]
    
    def __setitem__(self, key, value):
        self.dicts[-1][key] = value
    
    def __getitem__(self, key):
        for d in reversed(self.dicts):
            if key in d:
                return d[key]
        raise KeyError(key)
    
    def __delitem__(self, key):
        del self.dicts[-1][key]
    
    def __contains__(self, key):
        for d in self.dicts:
            if key in d:
                return True
        return False
    
    def push(self, _dict={}):
        self.dicts.append(_dict)
    
    def pop(self):
        self.dicts.pop()



from rivr.middleware import Middleware

try:
    import pymongo
except ImportError:
    pymongo = None

class mongodb(object):
    def __init__(self, func, database=True, collection=True, gridfs=False):
        self.func = func
        self.database = database
        self.collection = collection
        self.gridfs = gridfs
        
        if gridfs or collection:
            self.database = True
    
    def __call__(self, request, *args, **kwargs):
        if pymongo and 'mongodb_host' in kwargs and 'mongodb_port' in kwargs:
            request.mongodb_connection = pymongo.Connection(kwargs['mongodb_host'], kwargs['mongodb_port'])
            del kwargs['mongodb_host']
            del kwargs['mongodb_port']
        
        if 'mongodb_database' in kwargs and hasattr(request, 'mongodb_connection'):
            request.mongodb_database = request.mongodb_connection[kwargs['mongodb_database']]
            del kwargs['mongodb_database']
        
        if 'mongodb_collection' in kwargs and hasattr(request, 'mongodb_database'):
            request.mongodb_collection = request.mongodb_database[kwargs['mongodb_collection']]
            del kwargs['mongodb_collection']
        
        if self.database and not hasattr(request, 'mongodb_database'):
            raise Exception("MongoDB Database missing")
        
        if self.collection and not hasattr(request, 'mongodb_collection'):
            raise Exception("MongoDB Collection missing")
        
        if self.gridfs:
            import gridfs
            request.mongodb_gridfs = gridfs.GridFS(request.mongodb_database)
        
        return self.func(request, *args, **kwargs)

class MongoDBMiddleware(Middleware):
    def __init__(self, host=None, port=None, database=None, collection=None, handler=None, username=None, password=None):
        super(MongoDBMiddleware, self).__init__(handler)
        
        self.connection = None
        self.database = None
        self._database = None
        self.collection = None
        self._collection = None
        
        if pymongo:
            self.connection = pymongo.Connection(host, port)
        
        if database:
            self._database = database
            if self.connection:
                self.database = self.connection[database]
            if username and password:
                self.database.authenticate(username, password)
            
        if collection:
            self._collection = collection
            if self.connection:
                self.collection = self.database[collection]
    
    def process_request(self, request):
        if self.connection:
            request.mongodb_connection = self.connection
        
        if self.database:
            request.mongodb_database = self.database
        elif self._database and hasattr(request, 'mongodb_connection'):
            request.mongodb_database = request.mongodb_connection[self._database]
        
        if self.collection:
            request.mongodb_collection = self.collection
        elif self._collection and hasattr(request, 'mongodb_database'):
            request.mongodb_collection = request.mongodb_database[self._collection]



import os
import posixpath
import urllib
import mimetypes
import stat
import re
from email.Utils import formatdate, parsedate_tz, mktime_tz

from rivr.http import Response, ResponseRedirect, ResponseNotModified, Http404

def was_modified_since(header=None, mtime=0, size=0):
    if header is None:
        return True
    
    matches = re.match(r'^([^;]+)(; length=([0-9]+))?$', header, re.IGNORECASE)
    
    header_mtime = mktime_tz(parsedate_tz(matches.group(1)))
    header_len = matches.group(3)
    
    if header_len and int(header_len) != size:
        return True
    
    if mtime > header_mtime:
        return True
    
    return False

def serve(request, path, document_root=None, show_indexes=False):
    """
    Usage in the router:
    
    (r'(?P<path>.*)$', 'rivr.views.serve', {'document_root':'/usr/var/http/'})
    """
    
    # Clean up given path to only allow serving files below document_root.
    path = posixpath.normpath(urllib.unquote(path))
    path = path.lstrip('/')
    newpath = ''
    
    for part in path.split('/'):
        if not part:
            # Strip empty path components.
            continue
        
        drive, part = os.path.splitdrive(part)
        head, part = os.path.split(part)
        
        if part in (os.curdir, os.pardir):
            # Strip '.' and '..' in path.
            continue
        
        newpath = os.path.join(newpath, part).replace('\\', '/')
    
    if newpath and path != newpath:
        return ResponseRedirect(newpath)
    
    fullpath = os.path.join(document_root, newpath)
    
    if not os.path.exists(fullpath):
        raise Http404('"%s" does not exist.' % newpath)
    
    if os.path.isdir(fullpath):
        if show_indexes:
            return directory_index(request, newpath, fullpath)
        raise Http404('Directory indexes are not allowed here.')
    
    statobj = os.stat(fullpath)
    
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'), statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
        return ResponseNotModified()
    
    mimetype = mimetypes.guess_type(fullpath)[0] or 'application/octet-stream'
    contents = open(fullpath, 'rb').read()
    
    response = Response(contents, content_type=mimetype)
    response.headers['Last-Modified'] = '%s GMT' % formatdate(statobj[stat.ST_MTIME])[:25]
    response.headers['Content-Length'] = str(len(contents))
    
    return response

DEFAULT_DIRECTORY_INDEX_TEMPLATE = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
    <meta http-equiv="Content-type" content="text/html; charset=utf-8" />
    <meta http-equiv="Content-Language" content="en-us" />
    <meta name="robots" content="NONE,NOARCHIVE" />
    <title>Index of {{ directory }}</title>
</head>

<body>
    <h1>Index of {{ directory }}</h1>
    <ul>
      {% ifnotequal directory "/" %}
        <li><a href="../">../</a></li>
      {% endifnotequal %}
      
      {% for f in files %}
        <li><a href="{{ f }}">{{ f }}</a></li>
      {% endfor %}
    </ul>
</body>
"""

def directory_index(request, path, fullpath):
    from django.template import Context, Template
    
    t = Template(DEFAULT_DIRECTORY_INDEX_TEMPLATE)
    
    files = []
    for f in os.listdir(fullpath):
        if not f.startswith('.'):
            if os.path.isdir(os.path.join(fullpath, f)):
                f += '/'
            files.append(f)
    
    return Response(str(t.render(Context({
        'directory': path + '/',
        'files': files,
    }))))



def import_module(name):
    if callable(name):
        return name
    
    if not isinstance(name, str):
        raise ImportError('rivr.import.import_module cannot import %s because it is not a string.' % name)
    
    module = __import__(name)
    for i in name.split('.')[1:]:
        module = getattr(module, i)
    
    return module



from rivr import template

register = template.Library()

class IncludeNode(template.Node):
    def __init__(self, template_name):
        self.template_name = template_name
    
    def render(self, context):
        if 'loader' not in context:
            raise template.TemplateSyntaxError("Template loader not in context.")
        
        t = context['loader'].load_template(self.template_name)
        
        if isinstance(t, list):
            raise template.TemplateSyntaxError("%s template not found in: %s" % (self.template_name, ', '.join(t)))
        
        t = template.Template(t)
        return t.render(context)

def do_include(parser, token):
    bits = token.contents.split('"')
    
    if len(bits) != 3:
        raise template.TemplateSyntaxError("Include tag takes one argument: the template file to be included")
    
    return IncludeNode(bits[1])
register.tag('include', do_include)



import re

from rivr import template

register = template.Library()

class IfNode(template.Node):
    def __init__(self, var, nodelist_true=None, nodelist_false=None):
        self.var = template.Variable(var)
        self.nodelist_true, self.nodelist_false = nodelist_true, nodelist_false
    
    def render(self, context):
        try:
            if self.var.resolve(context):
                return self.nodelist_true.render(context)
            return self.nodelist_false.render(context)
        except template.VariableDoesNotExist:
            return self.nodelist_false.render(context)
    
def do_if(parser, token):
    var = token.split_contents()[1]
    nodelist_true = parser.parse(('else', 'endif'))
    token = parser.next_token()
    
    if token.contents == 'else':
        nodelist_false = parser.parse(('endif',))
        parser.delete_first_token()
    else:
        nodelist_false = template.NodeList()
    
    return IfNode(var, nodelist_true, nodelist_false)
register.tag('if', do_if)

def do_ifnot(parser, token):
    var = token.split_contents()[1]
    nodelist_false = parser.parse(('else', 'endif'))
    token = parser.next_token()
    
    if token.contents == 'else':
        nodelist_true = parser.parse(('endif',))
        parser.delete_first_token()
    else:
        nodelist_true = template.NodeList()
    
    return IfNode(var, nodelist_true, nodelist_false)
register.tag('ifnot', do_ifnot)

class IfEqualNode(template.Node):
    def __init__(self, var1, var2, nodelist_true=None, nodelist_false=None):
        self.var1 = template.Variable(var1)
        self.var2 = template.Variable(var2)
        self.nodelist_true, self.nodelist_false = nodelist_true, nodelist_false
    
    def render(self, context):
        try:
            if self.var1.resolve(context) == self.var2.resolve(context):
                return self.nodelist_true.render(context)
            return self.nodelist_false.render(context)
        except template.VariableDoesNotExist:
            return self.nodelist_false.render(context)

def do_ifequal(parser, token):
    var1 = token.split_contents()[1]
    var2 = token.split_contents()[2]
    nodelist_true = parser.parse(('else', 'endifequal'))
    token = parser.next_token()
    
    if token.contents == 'else':
        nodelist_false = parser.parse(('endifequal',))
        parser.delete_first_token()
    else:
        nodelist_false = template.NodeList()
    
    return IfEqualNode(var1, var2, nodelist_true, nodelist_false)
register.tag('ifequal', do_ifequal)

class ForNode(template.Node):
    def __init__(self, var, loopvars, nodelist_loop, nodelist_empty):
        self.var = template.Variable(var)
        self.loopvars = loopvars
        self.nodelist_loop = nodelist_loop
        self.nodelist_empty = nodelist_empty
    
    def render(self, context):
        try:
            values = self.var.resolve(context)
            if not hasattr(values, '__len__'):
                values = list(values)
        except KeyError:
            values = []
        
        if len(values) == 0:
            return self.nodelist_empty.render(context)
        
        result = []
        context.push()
        unpack = len(self.loopvars) > 1
        
        for item in values:
            if unpack:
                if (type(values) == dict) and (len(self.loopvars) == 2):
                    context.push(dict(zip(self.loopvars, [item, values[item]])))
                else:
                    context.push(dict(zip(self.loopvars, item)))
            else:
                context[self.loopvars[0]] = item
            result.append(self.nodelist_loop.render(context))
            if unpack:
                context.pop()
        
        context.pop()
        return ''.join(result)

def do_for(parser, token):
    bits = token.contents.split()
    if len(bits) < 4:
        raise template.TemplateSyntaxError("'for' statements should have atleast 4 works (words: %s)" % token.contents)
    
    if bits[-2] != 'in':
        raise template.TemplateSyntaxError("'for' statements should use the following 'for x in y': %s" % token.contents)
    
    var = bits[-1]
    loopvars = re.sub(r' *, *', ',', ' '.join(bits[1:-2])).split(',')
    
    nodelist_loop = parser.parse(('empty', 'endfor'))
    token = parser.next_token()
    if token.contents == 'empty':
        nodelist_empty = parser.parse(('endfor',))
        parser.delete_first_token()
    else:
        nodelist_empty = template.NodeList()
    
    return ForNode(var, loopvars, nodelist_loop, nodelist_empty)
register.tag('for', do_for)

class RenderNode(template.VariableNode):
    def render(self, context):
        try:
            return context[self.variable].render(context)
        except KeyError:
            return ''
        except AttributeError:
            return Template(context[self.variable]).render(context)

def do_render(parser, token):
    var = token.contents.split()[1]
    return RenderNode(var)
register.tag('render', do_render)

class SSINode(template.Node):
    def __init__(self, file_name):
        self.file_name = template.Variable(file_name)
    
    def render(self, context):
        try:
            fp = open(self.file_name.resolve(context), 'r')
            output = fp.read()
            fp.close()
        except IOError:
            output = ''
        
        return Template(output).render(context)

def do_ssi(parser, token):
    bits = token.contents.split()
    
    if len(bits) < 2:
        raise template.TemplateSyntaxError("ssi tag takes one argument: the path to the file to be included")
    
    return SSINode(bits[1])
register.tag('ssi', do_ssi)

class NowNode(template.Node):
    def __init__(self, format_string):
        self.format_string = format_string
    
    def render(self, context):
        from datetime import datetime
        return datetime.now().strftime(self.format_string)

def do_now(parser, token):
    bits = token.contents.split('"')
    if len(bits) != 3:
        raise template.TemplateSyntaxError("'now' statement takes one argument")
    format_string = bits[1]
    return NowNode(format_string)
register.tag('now', do_now)

def do_load(parser, token):
    bits = token.contents.split()
    
    if len(bits) < 2:
        raise template.TemplateSyntaxError("Load tag takes one argument: the module of the template library")
    
    lib = template.import_library(bits[1])
    parser.add_library(lib)
    return ''
register.tag('load', do_load)

class FilterNode(template.Node):
    def __init__(self, filter_expr, nodelist):
        self.filter_expr, self.nodelist = filter_expr, nodelist
    
    def render(self, context):
        output = self.nodelist.render(context)
        context.push({'var':output})
        filtered = self.filter_expr.resolve(context)
        context.pop()
        return filtered

def do_filter(parser, token):
    _, filter_expr = token.contents.split(None, 1)
    nodelist = parser.parse(('endfilter',))
    parser.delete_first_token()
    
    filter_expr = parser.compile_filter("var|%s" % filter_expr)
    return FilterNode(filter_expr, nodelist)
register.tag('filter', do_filter)



from rivr.http import ResponseNotAllowed, ResponseRedirect, ResponsePermanentRedirect
from rivr.template.response import TemplateResponse

class View(object):
    http_method_names = ['get', 'post', 'put', 'delete', 'head', 'options', 'trace']

    def __init__(self, **kwargs):
        for key, value in kwargs.iteritems():
            setattr(self, key, value)

    @classmethod
    def as_view(cls, **initkwargs):
        def view(*args, **kwargs):
            return cls(**initkwargs).dispatch(*args, **kwargs)
        return view

    def get_handler(self, request):
        if request.method.lower() in self.http_method_names:
            return getattr(self, request.method.lower(), self.http_method_not_allowed)
        return self.http_method_not_allowed

    def dispatch(self, request, *args, **kwargs):
        self.request = request
        self.args = args
        self.kwargs = kwargs

        return self.get_handler(request)(request, *args, **kwargs)

    def http_method_not_allowed(self, request, *args, **kwargs):
        allowed_methods = [m for m in self.http_method_names if hasattr(self, m)]
        return ResponseNotAllowed(allowed_methods)

class RedirectView(View):
    permanent = True
    url = None
    query_string = False

    def get_redirect_url(self, **kwargs):
        if self.url:
            args = self.request.META["QUERY_STRING"]
            if args and self.query_string:
                url = "%s?%s" % (self.url, args)
            else:
                url = self.url
            return self.url % kwargs
        return None

    def get(self, request, *args, **kwargs):
        url = self.get_redirect_url(**kwargs)
        if url:
            if self.permanent:
                return ResponsePermanentRedirect(url)
            return ResponseRedirect(url)
        raise Http404("Redirect URL not found.")

class TemplateMixin(object):
    template_name = None
    response_class = TemplateResponse

    def get_template_names(self):
        if self.template_name is None:
            raise Exception("TemplateResponseMixin requires either a"
                            "definition of 'template_name' or a"
                            "implementation of 'get_template_names()'")
        return [self.template_name]

    def render_to_response(self, context):
        return self.response_class(self.request, self.get_template_names(), context)

class TemplateView(TemplateMixin, View):
    def get_context_data(self, **kwargs):
        return {'params': kwargs}

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)




from rivr import template

register = template.Library()

def escape(value):
    return template.html_escape(value)
register.filter('escape', escape)

def first(value):
    try:
        return value[0]
    except IndexError:
        return ''
register.filter('first', first)

def last(value):
    try:
        return value[-1]
    except IndexError:
        return ''
register.filter('last', last)

def capitalize(value):
    return value.capitalize()
register.filter('capitalize', capitalize)



from rivr.http import Response
from rivr.template import Context
from rivr.template.loader import Loader

class TemplateResponse(Response):
    def __init__(self, request, template_name, context, template_dirs=[], *args, **kwargs):
        self.request = request
        self.loader = None
        self.template_name = template_name
        self.context = context
        self.template_dirs = template_dirs
        self._content = None
        super(TemplateResponse, self).__init__(*args, **kwargs)
    
    def resolve_template(self):
        self.loader = Loader()
        
        if isinstance(self.template_dirs, list):
            self.loader.template_dirs += self.template_dirs
        else:
            self.loader.template_dirs.append(self.template_dirs)
        
        return self.loader.get_template(self.template_name)
    
    def resolve_context(self):
        if isinstance(self.context, Context):
            return self.context
        
        c = Context({'request':self.request, 'loader':self.loader})
        c.push(self.context)
        self.context = c
        
        return self.context
    
    def render(self):
        t = self.resolve_template()
        c = self.resolve_context()
        return t.render(c)
    
    def get_content(self):
        if self._content:
            return self._content
        
        self._content = self.render()
        return self._content
    
    def set_content(self, value):
        pass
    content = property(get_content, set_content)

class TemplateMiddleware(object):
    def __init__(self, template_dirs, view):
        self.template_dirs = template_dirs
        self.view = view
    
    def __call__(self, *args, **kwargs):
        response = self.view(*args, **kwargs)
        
        if isinstance(response, TemplateResponse):
            response.template_dirs = self.template_dirs
        
        return response

def direct_to_template(request, template, extra_context={}, **kwargs):
    return TemplateResponse(request, template, extra_context)



import re

from rivr.importlib import import_module
from rivr.template.context import Context

__all__ = ('Template', 'Context')

HTML_ESCAPE_DICT = {
    '<': '&lt;',
    '>': '&gt;',
    
    '&':'&amp;',
    '"':'&quot;',
    "'":'&#39;', 
}

FILTER_SEPARATOR = '|'
FILTER_ARGUMENT_SEPARATOR = ':'
VARIABLE_ATTRIBUTE_SEPARATOR = '.'
BLOCK_TAG_START = '{%'
BLOCK_TAG_END = '%}'
VARIABLE_TAG_START = '{{'
VARIABLE_TAG_END = '}}'
COMMENT_TAG_START = '{#'
COMMENT_TAG_END = '#}'

TOKEN_TEXT = 0
TOKEN_VAR = 1
TOKEN_BLOCK = 2
TOKEN_COMMENT = 3

tag_re = re.compile('(%s.*?%s|%s.*?%s|%s.*?%s)' % (
    re.escape(BLOCK_TAG_START), re.escape(BLOCK_TAG_END),
    re.escape(VARIABLE_TAG_START), re.escape(VARIABLE_TAG_END),
    re.escape(COMMENT_TAG_START), re.escape(COMMENT_TAG_END))
)

literal_re = re.compile(r'^(\'.*\'|".*")$')

def import_library(library_module):
    module = import_module(library_module)
    return getattr(module, 'register')

class TemplateSyntaxError(Exception):
    pass

class VariableDoesNotExist(Exception):
    pass

class Variable(object):
    def __init__(self, var):
        self.var = var
        self.lookups = None
        self.literal = ''
        
        try:
            self.literal = float(var)
            if '.' not in var and 'e' not in var.lower():
                self.literal = int(var)
        except ValueError:
            if literal_re.match(var):
                self.literal = var[1:-1]
            else:
                self.lookups = tuple(var.split(VARIABLE_ATTRIBUTE_SEPARATOR))
    
    def resolve(self, context, fail_silently=True):
        if self.lookups is not None:
            try:
                return self.resolve_lookup(context)
            except VariableDoesNotExist:
                if fail_silently:
                    return ''
                else:
                    raise
        return self.literal
    
    def resolve_lookup(self, context):
        current = context
        for bit in self.lookups:
            try: # Dictionary lookup
                current = current[bit]
            except (TypeError, AttributeError, KeyError):
                try: # Attribute lookup
                    current = getattr(current, bit)
                    if callable(current):
                        try:
                            current = current()
                        except TypeError: # This function requires arguments
                            raise VariableDoesNotExist
                except (TypeError, AttributeError):
                    try: # list-index lookup
                        current = current[int(bit)]
                    except (
                        IndexError, # list index out of range
                        ValueError, # invalid literal for int()
                        KeyError,   # current is a dict without `int(bit)` key
                        TypeError,  # unsubscriptable object
                    ):
                       raise VariableDoesNotExist
        return current

class FilterExpression(object):
    def __init__(self, token, parser):
        self.token = token
        self.filters = []
        bits = token.split('|')
        
        if len(bits) < 1:
            raise TemplateSyntaxError("Variable tags must inclue 1 argument")
        
        self.var = Variable(bits[0].strip())
        
        for filter_name in bits[1:]:
            self.filters.append(parser.find_filter(filter_name.strip()))
    
    def resolve(self, context):
        if isinstance(self.var, Variable):
            obj = self.var.resolve(context)
        else:
            obj = self.var
        
        for filter_func in self.filters:
            obj = filter_func(obj)
        
        return obj

class NodeList(list):
    def render(self, context):
        bits = []
        for node in self:
            if isinstance(node, Node):
                bits.append(node.render(context))
            else:
                bits.append(node)
        return ''.join(bits)

class Node(object):
    def render(self, context):
        pass

class TextNode(Node):
    def __init__(self, s):
        self.s = s
    
    def render(self, context):
        return self.s

class VariableNode(Node):
    def __init__(self, s):
        self.variable = s
    
    def render(self, context):
        return unicode(self.variable.resolve(context))

class Token(object):
    def __init__(self, token_type, contents):
        self.token_type, self.contents = token_type, contents
    
    def split_contents(self):
        return self.contents.split()

class Lexer(object):
    def __init__(self, template_string):
        self.template_string = template_string
    
    def tokenize(self):
        nodes = []
        
        for bit in tag_re.split(self.template_string):
            if bit:
                nodes.append(self.create_token(bit))
        
        return nodes
    
    def create_token(self, token_string):
        if token_string.startswith(VARIABLE_TAG_START):
            return Token(TOKEN_VAR, token_string[len(VARIABLE_TAG_START):-len(VARIABLE_TAG_END)].strip())
        elif token_string.startswith(BLOCK_TAG_START):
            return Token(TOKEN_BLOCK, token_string[len(BLOCK_TAG_START):-len(BLOCK_TAG_END)].strip())
        elif token_string.startswith(COMMENT_TAG_START):
            return Token(TOKEN_COMMENT, '')
        return Token(TOKEN_TEXT, token_string)

class Parser(object):
    def __init__(self, tokens):
        self.tokens = tokens
        self.tags = {}
        self.filters = {}
        self.add_library(import_library('rivr.template.defaulttags'))
        self.add_library(import_library('rivr.template.defaultfilters'))
        self.add_library(import_library('rivr.template.loader_tags'))
    
    def add_library(self, lib):
        self.tags.update(lib.tags)
        self.filters.update(lib.filters)
    
    def parse(self, parse_until=[]):
        nodes = NodeList()
        while self.tokens:
            token = self.next_token()
            if token.token_type == TOKEN_TEXT:
                nodes.append(TextNode(token.contents))
            elif token.token_type == TOKEN_VAR:
                nodes.append(VariableNode(self.compile_filter(token.contents)))
            elif token.token_type == TOKEN_BLOCK:
                if token.contents in parse_until:
                    self.prepend_token(token)
                    return nodes
                
                try:
                    command = token.split_contents()[0]
                except IndexError:
                    command = ''
                
                if command in self.tags:
                    nodes.append(self.tags[command](self, token))
                else:
                    raise TemplateSyntaxError("No such tag %s" % command)
        
        return nodes
    
    def compile_filter(self, token):
        return FilterExpression(token, self)
    
    def find_filter(self, filter_name):
        if filter_name in self.filters:
            return self.filters[filter_name]
        else:
            raise TemplateSyntaxError("Invalid filter: '%s'" % filter_name)
    
    def prepend_token(self, token):
        self.tokens.insert(0, token)
    
    def next_token(self):
        return self.tokens.pop(0)
    
    def delete_first_token(self):
        del self.tokens[0]

class Template(object):
    def __init__(self, template_string):
        self.nodelist = template_string_to_nodes(template_string)
    
    def render(self, context):
        try:
            return self.nodelist.render(context)
        except TemplateSyntaxError, e:
            e.template = self
            e.context = context
            raise e

class Library(object):
    def __init__(self):
        self.tags = {}
        self.filters = {}
    
    def tag(self, name, function):
        self.tags[name] = function
    
    def filter(self, name, function):
        self.filters[name] = function

def template_string_to_nodes(template_string):
    tokens = Lexer(template_string).tokenize()
    parser = Parser(tokens)
    return parser.parse()

def html_escape(text):
    for escape in HTML_ESCAPE_DICT:
        text = text.replace(escape, HTML_ESCAPE_DICT[escape])
    
    return text



from base64 import b64decode

try:
    from hashlib import sha256
except ImportError: # Python < 2.5
    try:
        from Crypto.Hash import SHA256
        sha256 = SHA256.new
    except ImportError:
        print("Update to python 2.5+, or install PyCrypto.")
        raise ImportError("No SHA256")

from rivr.http import Response

class ResponseAuthorizationRequired(Response):
    status_code = 401
    
    def __init__(self, *args, **kwargs):
        super(ResponseAuthorizationRequired, self).__init__(*args, **kwargs)
        self.headers['WWW-Authenticate'] = 'Basic realm="Secure Area"'

class User(object):
    def __init__(self, username=''):
        self.username = username
    
    def is_anonymous(self):
        return False
    
    def is_authenticated(self):
        return True

class AnnonymousUser(object):
    username = ''
    
    def is_anonymous(self):
        return True

    def is_authenticated(self):
        return False

class AuthMiddleware(object):
    def __init__(self, handler, view, needs_auth=True):
        self.handler = handler
        self.view = view
        self.needs_auth = needs_auth
    
    def __call__(self, request, *args, **kwargs):
        request.user = self.check_login(request)
        
        if not request.user.is_authenticated() and self.needs_auth:
            return ResponseAuthorizationRequired()
        
        return self.view(request, *args, **kwargs)
    
    def hash(self, password):
        return sha256(password).hexdigest()
    
    def check_login(self, request):
        authorization = request.META.get('HTTP_AUTHORIZATION', False)
        
        if not authorization:
            return AnnonymousUser()
        
        try:
            method, key = authorization.split()
        except ValueError:
            return AnnonymousUser()
        
        if method != 'Basic':
            return AnnonymousUser()
        
        try:
            username, password = b64decode(key).split(':')
        except ValueError:
            return AnnonymousUser()
        
        if self.handler(username, password):
            return User(username)
        
        return AnnonymousUser()



from rivr.http import ResponseRedirect, ResponsePermanentRedirect

class RedirectMiddleware(object):
    def __init__(self, url, permanent=False):
        self.url = url
        self.permanent = permanent
    
    def __call__(self, request, **kwargs):
        if self.permanent:
            return ResponsePermanentRedirect(self.url % kwargs)
        return ResponseRedirect(self.url % kwargs)



try:
    # The mod_python version is more efficient, so try importing it first.
    from mod_python.util import parse_qsl
except ImportError:
    from cgi import parse_qsl

from rivr.http import parse_cookie, Request, Response, ResponseNotFound, Http404

STATUS_CODES = {
    100: 'CONTINUE',
    101: 'SWITCHING PROTOCOLS',
    200: 'OK',
    201: 'CREATED',
    202: 'ACCEPTED',
    203: 'NON-AUTHORITATIVE INFORMATION',
    204: 'NO CONTENT',
    205: 'RESET CONTENT',
    206: 'PARTIAL CONTENT',
    300: 'MULTIPLE CHOICES',
    301: 'MOVED PERMANENTLY',
    302: 'FOUND',
    303: 'SEE OTHER',
    304: 'NOT MODIFIED',
    305: 'USE PROXY',
    306: 'RESERVED',
    307: 'TEMPORARY REDIRECT',
    400: 'BAD REQUEST',
    401: 'UNAUTHORIZED',
    402: 'PAYMENT REQUIRED',
    403: 'FORBIDDEN',
    404: 'NOT FOUND',
    405: 'METHOD NOT ALLOWED',
    406: 'NOT ACCEPTABLE',
    407: 'PROXY AUTHENTICATION REQUIRED',
    408: 'REQUEST TIMEOUT',
    409: 'CONFLICT',
    410: 'GONE',
    411: 'LENGTH REQUIRED',
    412: 'PRECONDITION FAILED',
    413: 'REQUEST ENTITY TOO LARGE',
    414: 'REQUEST-URI TOO LONG',
    415: 'UNSUPPORTED MEDIA TYPE',
    416: 'REQUESTED RANGE NOT SATISFIABLE',
    417: 'EXPECTATION FAILED',
    500: 'INTERNAL SERVER ERROR',
    501: 'NOT IMPLEMENTED',
    502: 'BAD GATEWAY',
    503: 'SERVICE UNAVAILABLE',
    504: 'GATEWAY TIMEOUT',
    505: 'HTTP VERSION NOT SUPPORTED',
}

class WSGIRequest(object):
    def __init__(self, environ):
        self.environ = environ
        
        self.method = environ.get('REQUEST_METHOD', 'GET').upper()
        self.path = environ.get('PATH_INFO', '/')
        self.META = environ
    
    #@property
    def get_get(self):
        if not hasattr(self, '_get'):
            self._get = dict((k,v) for k,v in parse_qsl(self.environ.get('QUERY_STRING', ''), True))
        return self._get
    GET = property(get_get)
    
    #@property
    def get_post(self):
        if not hasattr(self, '_post'):
            try:
                content_length = int(self.environ.get('CONTENT_LENGTH', 0))
            except (ValueError, TypeError):
                content_length = 0
            
            if content_length > 0:
                body = self.environ['wsgi.input'].read(int(self.environ.get('CONTENT_LENGTH', 0)))
                self._post = dict((k,v) for k,v in parse_qsl(body, True))
            else:
                self._post = {}
        return self._post
    POST = property(get_post)
    
    #@property
    def get_cookies(self):
        if not hasattr(self, '_cookies'):
            self._cookies = parse_cookie(self.environ.get('HTTP_COOKIE', ''))
        return self._cookies
    COOKIES = property(get_cookies)

class WSGIHandler(object):
    request_class = WSGIRequest
    
    def __init__(self, view):
        self.view = view
    
    def __call__(self, environ, start_response):
        request = self.request_class(environ)
        
        try:
            response = self.view(request)
            if not response:
                raise Exception("View did not return a response.")
        except Http404, e:
            response = ResponseNotFound('Page not found')
        except Exception, e:
            response = Response('Internal server error', status=500)
        
        try:
            status_text = STATUS_CODES[response.status_code]
        except KeyError:
            status_text = 'UNKNOWN STATUS CODE'
        
        status = '%s %s' % (response.status_code, status_text)
        
        start_response(status, response.headers_items())
        
        if isinstance(response.content, unicode):
            return [response.content.encode('utf-8')]        
        return [response.content]



import math

from rivr.template.response import TemplateResponse
from rivr.http import Response, ResponseRedirect, Http404
from rivr.middleware.mongodb import mongodb

try:
    from pymongo.objectid import ObjectId
    from pymongo import json_util
    import gridfs
except ImportError:
    ObjectId = lambda x: x
    gridfs = None

class GridFSResponse(Response):
    def __init__(self, grid_file):
        super(GridFSResponse, self).__init__(content_type=str(grid_file.content_type))
        self.grid_file = grid_file
    
    def get_content(self):
        if not hasattr(self, '_content'):
            self._content = self.grid_file.read()
        return self._content
    
    def set_content(self, value):
        pass
    content = property(get_content, set_content)

#@mongodb
def gridfs_view(request, object_id):
    f = request.mongodb_gridfs.get(ObjectId(object_id))
    return GridFSResponse(f)
gridfs_view = mongodb(gridfs_view, collection=False, gridfs=True)

def object_lookup(func):
    def new_func(request, object_id=None, slug=None, slug_field='slug', *args, **kwargs):
        lookup = {}
        
        if object_id:
            lookup['_id'] = ObjectId(object_id)
        
        if slug:
            lookup[slug_field] = slug
        
        return func(request, lookup, *args, **kwargs)
    return new_func

lookup_filters = {
    'int': int,
    'gt': lambda x: {'$gt': int(x)},
    'gte': lambda x: {'$gte': int(x)},
    'lt': lambda x: {'$lt': int(x)},
    'lte': lambda x: {'$lte': int(x)},
    'regex': lambda x: {'$regex': x},
    'iregex': lambda x: {'$regex': x, '$options': 'i'},
}

#@mongodb
def object_list(request, page=1, paginate_by=20, template_name=None, template_object_name='object', serializable=False):
    if not template_name:
        template_name = ['%s_list.html' % template_object_name]
        
        if template_object_name == 'object':
            template_name.insert(0, '%s_list.html' % request.mongodb_collection.name)
    
    lookup = {}
    output = 'template'
    
    for l in request.GET:
        if l == 'page':
            page = request.GET[l]
        elif l == 'o':
            output = request.GET[l]
        elif '__' in l:
            try:
                key, f = l.split('__')
                lookup[key] = lookup_filters[f](request.GET[l])
            except:
                continue
        else:
            lookup[l] = request.GET[l]
    
    try:
        page = int(page)
        if page < 1:
            page = 1
    except (TypeError, ValueError):
        page = 1
    
    query = request.mongodb_collection.find(lookup)
    paged_query = query.skip(paginate_by * (page - 1)).limit(paginate_by)
    
    if output == 'json' and serializable:
        import json
        serialized = json.dumps([x for x in paged_query], default=json_util.default)
        return Response(serialized, content_type='application/json')
    
    return TemplateResponse(request, template_name, {
        '%s_list' % template_object_name: paged_query,
        'mongodb_collection': request.mongodb_collection.name,
        # Pagination
        'page': page,
        'paginate_by': paginate_by,
        'page_count': int(math.ceil(float(query.count()) / float(paginate_by)))
    })
object_list = mongodb(object_list)

#@mongodb
def object_detail(request, lookup, template_name=None, template_object_name='object'):
    if not template_name:
        template_name = ['%s_detail.html' % template_object_name]
        
        if template_object_name == 'object':
            template_name.insert(0, '%s_detail.html' % request.mongodb_collection.name)
    
    return TemplateResponse(request, template_name, {
        template_object_name: request.mongodb_collection.find_one(lookup),
        'mongodb_collection': request.mongodb_collection.name
    })

object_detail = object_lookup(object_detail)
object_detail = mongodb(object_detail)

#@mongodb
def delete_object(request, lookup, template_name=None, template_object_name='object', post_delete_redirect='/'):
    obj = request.mongodb_collection.find_one(lookup)
    
    if not obj:
        raise Http404
    
    if request.method == 'POST':
        request.mongodb_collection.remove(lookup)
        return ResponseRedirect(post_delete_redirect)
    
    if not template_name:
        template_name = ['%s_confirm_delete.html' % template_object_name]
        
        if template_object_name == 'object':
            template_name.insert(0, '%s_confirm_delete.html' % request.mongodb_collection.name)
    
    return TemplateResponse(request, template_name, {
        template_object_name: obj,
        'mongodb_collection': request.mongodb_collection.name
    })

delete_object = object_lookup(delete_object)
delete_object = mongodb(delete_object)



from os.path import join

from rivr import template

class TemplateLoadError(Exception):
    def __init__(self, template, tried):
        self.template = template
        self.tried = tried
    
    def __str__(self):
        return 'Unable to load %s' % self.template

class Loader(object):
    def __init__(self):
        self.template_dirs = []
    
    def load_template(self, template_names):
        tried = []
        
        if isinstance(template_names, str):
            template_names = [template_names]
        
        for template_name in template_names:
            for template_dir in self.template_dirs:
                filepath = join(template_dir, template_name)
                try:
                    f = open(filepath)
                    try:
                        return f.read()
                    finally:
                        f.close()
                except IOError:
                    tried.append(filepath)
            
        return tried
    
    def get_template(self, template_name):
        template_content = self.load_template(template_name)
        
        if isinstance(template_content, list):
            raise TemplateLoadError(template_name, template_content)
        
        return template.Template(template_content)



from Cookie import SimpleCookie, CookieError

def parse_cookie(cookie):
    if cookie == '':
        return {}
    
    try:
        c = SimpleCookie()
        c.load(cookie)
    except CookieError:
        return {}
    
    cookiedict = {}
    for key in c.keys():
        cookiedict[key] = c.get(key).value
    
    return cookiedict

class Http404(Exception): pass

class Request(object):
    def __init__(self, path='/', method='GET', GET={}, POST={}, COOKIES={}):
        self.path = path
        self.method = method
        self.GET = GET
        self.POST = POST
        self.COOKIES = COOKIES

class Response(object):
    status_code = 200
    
    def __init__(self, content='', status=None, content_type='text/html; charset=utf8'):
        self.content = content
        
        if status:
            self.status_code = status
        
        self.headers = {'Content-Type': content_type}
        self.cookies = SimpleCookie()
    
    def __str__(self):
        return '\n'.join(['%s: %s' % (key, value) for key, value in self.headers.items()]) + '\n\n' + self.content
    
    def set_cookie(self, key, value='', max_age=None, expires=None, path='/', domain=None, secure=False):
        self.cookies[key] = value
        
        if max_age is not None:
            self.cookies[key]['max-age'] = max_age
        
        if expires is not None:
            self.cookies[key]['expires'] = expires
        
        if path is not None:
            self.cookies[key]['path'] = path
        
        if domain is not None:
            self.cookies[key]['domain'] = domain
        
        if secure:
            self.cookies[key]['secure'] = True
    
    def delete_cookie(self, key, path='/', domain=None):
        self.set_cookie(key, max_age=0, path=path, domain=domain, expires='Thu, 01-Jan-1970 00:00:00 GMT')
    
    def headers_items(self):
        headers = self.headers.items()
        
        for cookie in self.cookies.values():
            headers.append(('Set-Cookie', str(cookie.output(header=''))))
        
        return headers

class ResponseRedirect(Response):
    status_code = 302
    
    def __init__(self, redirect_to):
        super(ResponseRedirect, self).__init__()
        self.headers['Location'] = str(redirect_to)

class ResponsePermanentRedirect(ResponseRedirect):
    status_code = 301

class ResponseNotFound(Response):
    status_code = 404

class ResponseNotModified(Response):
    status_code = 304

class ResponseNotAllowed(Response):
    status_code = 405

    def __init__(self, permitted_methods):
        super(ResponseNotAllowed, self).__init__()
        self.headers['Allow'] = ', '.join(permitted_methods)




class Middleware(object):
    def __init__(self, handler=None):
        self.handler = handler

    def __call__(self, request):
        if hasattr(self, 'process_request'):
            response = self.process_request(request)

            if response:
                return response

        try:
            response = self.handler(request)
        except Exception, e:
            if hasattr(self, 'process_exception'):
                response = self.process_exception(request, e)

                if response:
                    return response
            else:
                raise e

        if hasattr(self, 'process_response'):
            response = self.process_response(request, response)

        return response


class MiddlewareController(Middleware):
    def __init__(self, handler, *middleware):
        super(MiddlewareController, self).__init__(handler)
        self.request_middleware = []
        self.response_middleware = []
        self.exception_middleware = []

        for mw_instance in middleware:
            self.append(mw_instance)

    def append(self, middleware):
        if hasattr(middleware, 'process_request'):
            self.request_middleware.append(middleware.process_request)
        if hasattr(middleware, 'process_response'):
            self.response_middleware.insert(0, middleware.process_response)
        if hasattr(middleware, 'process_exception'):
            self.exception_middleware.insert(0, middleware.process_exception)

    def process_request(self, request):
        response = None

        for request_mw in self.request_middleware:
            response = request_mw(request)
            if response:
                break

        if response is None:
            try:
                response = self.handler(request)
            except Exception, e:
                for exception_mw in self.exception_middleware:
                    response = exception_mw(request, e)
                    if response:
                        return response
                else:
                    raise e

        for response_mw in self.response_middleware:
            response = response_mw(request, response)

        return response

    def process_response(self, request, response):
        pass

    def process_exception(self, request, exception):
        pass



#!/usr/bin/env python

from distutils.core import setup

from rivr import VERSION

setup(
    name='rivr',
    version=VERSION,
    author='Kyle Fuller',
    author_email='inbox@kylefuller.co.uk',
    url='http://github.com/kylef/rivr/',
    download_url='http://github.com/kylef/rivr/zipball/%s' % VERSION,
    packages=['rivr', 'rivr.middleware', 'rivr.views', 'rivr.template'],
    license='BSD',
    summary='rivr is a microweb framework inspired by djng',
    description="rivr is a microweb framework inspired by djng, the reason I decided to create rivr and not use djng was that djng still depended on Django. I wanted rivr for places where I don't have Django. It is a lightweight framework which can be included along side another python application. rivr does not have a database layer, you are free to use whatever you choose.",
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ]
)



VERSION = '0.2.1'

from rivr.http import Response, Http404
from rivr.server import serve

from rivr.middleware import Middleware, MiddlewareController
from rivr.middleware.debug import DebugMiddleware
from rivr.middleware.auth import AuthMiddleware
from rivr.middleware.mongodb import MongoDBMiddleware, mongodb
from rivr.middleware.redirect import RedirectMiddleware

from rivr.router import Router, Domain, url, include
from rivr.array import Array

from rivr.template import Template, Context
from rivr.template.response import TemplateResponse, TemplateMiddleware, direct_to_template

from rivr.views.base import View, TemplateView



import sys

from rivr.middleware import Middleware
from rivr.http import Response, ResponseNotFound, Http404
from rivr.template import html_escape

ERROR_HTML = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
    <meta http-equiv="Content-type" content="text/html; charset=utf-8" />
    <title>%(title)s</title>
    <style type="text/css" media="screen">
    html * { padding:0; margin:0; }
    body * { padding:10px 20px; font-family: 'Helvetica Neue', HelveticaNeue, Arial, Helvetica, sans-serif; color: #555; }
    body * * { padding:0; }
    h1, h2 { font-weight: normal; margin-bottom:.4em; letter-spacing: -1px; }
    h1 { font-size: 27px; }
    h2 { font-size: 22px; }
    #header { background: #eeeeee; border-bottom: 1px solid #ddd; }
    #header h1 { text-shadow: #fff 2px 2px 0; }
    #content { margin: 5px; }
    pre { border: 1px solid #ddd; margin: 10px 0; background: #eee; width: 720px; padding: 10px; border-radius: 6px; overflow: auto; font-size: 9pt; font-weight: normal; }
    code { font-family: Consolas, Monaco, 'Lucida Console', 'Liberation Mono', 'DejaVu Sans Mono', 'Bitstream Vera Sans Mono', 'Courier New'; }
    </style>
</head>
<body>
    <div id="header">
        <h1>%(title)s</h1>
    </div>
    
    <div id="content">
        <h2>Traceback</h2>
        <pre><code>%(traceback)s</code></pre>
        
        <h2>Request</h2>
        <pre><code>%(request)s</code></pre>
    </div>
</body>
"""

class DebugMiddleware(Middleware):
    def process_404(self, request):
        return ResponseNotFound('Page Not Found')
    
    def process_exception(self, request, e):
        if isinstance(e, Http404):
            return self.process_404(request)
        
        import traceback
        tb = '\n'.join(traceback.format_exception(*sys.exc_info()))

        html = ERROR_HTML % {
            'title':str(e),
            'traceback': tb,
            'request':html_escape(repr(request))
        }

        return Response(html, status=500)
    
    def process_response(self, request, response):
        if not response:
            return self.process_exception(request, Exception, "View did not return a response.")
        return response



import re

from rivr.importlib import import_module
from rivr.http import Http404
from rivr.http import ResponsePermanentRedirect

class Resolver404(Http404): pass
class ViewDoesNotExist(Exception): pass

class RegexURL(object):
    def __init__(self, regex):
        self.regex = re.compile(regex, re.UNICODE)
    
    def resolve(self, path):
        match = self.regex.search(path)
        if match:
            return self.match_found(path, match)

class RegexURLPattern(RegexURL):
    def __init__(self, regex, callback, kwargs={}, name=None, prefix=None):
        super(RegexURLPattern, self).__init__(regex)
        
        if callable(callback):
            self._callback = callback
        else:
            self._callback = None
            self._callback_str = callback
        
        self.default_kwargs = kwargs
        self.name = name
        
        self.add_prefix(prefix)
    
    def match_found(self, path, match):
        kwargs = match.groupdict()
        
        if kwargs:
            args = tuple()
        else:
            args = match.groups()
        
        kwargs.update(self.default_kwargs)
        
        return self.callback, args, kwargs
    
    #@property
    def callback(self):
        if self._callback is not None:
            return self._callback
        try:
            self._callback = import_module(self._callback_str)
        except ImportError, e:
            raise ViewDoesNotExist('Could not import %s. Error was: %s' % (self._callback_str, str(e)))
        
        return self._callback
    callback = property(callback)
    
    def add_prefix(self, prefix):
        if prefix and hasattr(self, '_callback_str'):
            self._callback_str = prefix + '.' + self._callback_str

class RegexURLResolver(RegexURL):
    def __init__(self, regex, router, kwargs={}):
        super(RegexURLResolver, self).__init__(regex)
        
        if callable(router):
            self._router = router
        else:
            self._router = None
            self._router_str = router
        
        self.default_kwargs = kwargs
    
    def match_found(self, path, match):
        new_path = path[match.end():]
        
        try:
            callback, args, kwargs = self.router.resolve(new_path)
        except Http404:
            return
        
        kwargs.update(self.default_kwargs)
        return callback, args, kwargs
    
    #@property 
    def router(self):
        if self._router is not None:
            return self._router
        try:
            self._router = import_module(self._router_str)
        except ImportError, e:
            raise Http404('Could not import %s. Error was: %s' % (mod_name, str(e)))
        
        return self._router
    router = property(router)

class BaseRouter(object):
    def __init__(self, *urls):
        self.urlpatterns = []
        
        try:
            if isinstance(urls[0], str):
                prefix = urls[0]
                urls = urls[1:]
            else:
                prefix = None
        except IndexError:
            prefix = None
        
        for t in urls:
            if isinstance(t, (list, tuple)):
                t = url(*t)
            
            if prefix and hasattr(t, 'add_prefix'):
                t.add_prefix(prefix)
            
            self.urlpatterns.append(t)
    
    def __iadd__(self, router):
        self.urlpatterns.__iadd__(router.urlpatterns)
    
    def __isub__(self, router):
        self.urlpatterns.__isub__(router.urlpatterns)
    
    def append(self, url):
        self.urlpatterns.append(url)
    
    def register(self, *t):
        if isinstance(t, (list, tuple)):
            t = url(*t)
        
        self.append(t)

class Router(BaseRouter):
    APPEND_SLASH = True
    
    def resolve(self, path):
        for pattern in self.urlpatterns:
            result = pattern.resolve(path)
            if result is not None:
                return result
        raise Resolver404('No URL pattern matched.')
    
    def __call__(self, request):
        if self.APPEND_SLASH and (not request.path.endswith('/')):
            if (not self.is_valid_path(request.path)) and self.is_valid_path(request.path+'/'):
                return ResponsePermanentRedirect(request.path+'/')
            
        result = RegexURLResolver(r'^/', self).resolve(request.path)
        if result:
            callback, args, kwargs = result
            return callback(request, *args, **kwargs)
        raise Resolver404()
    
    def is_valid_path(self, path):
        try:
            self.resolve(path)
            return True
        except Resolver404:
            return False

class Domain(BaseRouter):
    APPEND_SLASH = True
    
    def resolve(self, path):
        for pattern in self.urlpatterns:
            result = pattern.resolve(path)
            if result is not None:
                return result
        raise Resolver404('No URL pattern matched.')
    
    def __call__(self, request):
        host = request.META.get('HTTP_HOST', 'localhost:80')
        host = ':'.join(host.split(':')[:-1])
        url = host + request.path
        
        if self.APPEND_SLASH and (not url.endswith('/')):
            if (not self.is_valid_url(url)) and self.is_valid_url(url+'/'):
                return ResponsePermanentRedirect(url+'/')
        
        result = self.resolve(url)
        if result:
            callback, args, kwargs = result
            return callback(request, *args, **kwargs)
        raise Resolver404()
    
    def is_valid_url(self, path):
        try:
            self.resolve(path)
            return True
        except Resolver404:
            return False

# Shortcuts
def url(regex, view, kwargs={}, name=None, prefix=None):
    if isinstance(view, list):
        return RegexURLResolver(regex, view[0], kwargs)
    else:
        return RegexURLPattern(regex, view, kwargs, name, prefix)

def include(router):
    return [router]
